//@author: dottore
//@description: draws a Constant ParticlesGPU Sprites mesh using the DataTexture
//@tags: particles sprites
//@credits: Viktor Vicsek for Sprites Function

// --------------------------------------------------------------------------------------------------
// PARAMETERS:
// --------------------------------------------------------------------------------------------------

//transforms
float4x4 tW: WORLD ;
float4x4 tP: PROJECTION ;
float4x4 tWVP: WORLDVIEWPROJECTION ;
float4x4 tVP: VIEWPROJECTION ;


texture TexData <string uiname="Data Texture";>;
texture Tex <string uiname="TextureForQuads";>;

float alfaMultiple <string uiname="AlfaMultiple";>;
float4 colorMultiple: COLOR ;


sampler SampData = sampler_state    //sampler for doing the texture-lookup
{
    Texture   = (TexData);          //apply a texture to the sampler
    MipFilter = none;                    //sampler states
    MinFilter = none;
    MagFilter = none;
};

/////connect this input to the BackBufferHeight output of the renderer
float ViewportHeight;
/////decide if you want perspective
bool calcPerspective<string uiname="Calculate Scale From Perspective";>;
//yet to be done: allow rotating the TexCoords coz the quads cant be rotated
//float rotation<string uiname="Rotate Texcoords";>;

	//	texture for every quad
sampler spriteTextureSamp = sampler_state
{
    Texture   = (Tex);
    MipFilter = LINEAR;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
	AddressU = Clamp;
	AddressV = Clamp;
};


// --------------------------------------------------------------------------------------------------
// --------------------------------------------------------------------------------------------------
// --------------------------------------------------------------------------------------------------

struct vs2ps
{
    float4 Pos : POSITION ;
    float2 TexCd2 : TEXCOORD0 ;
    float Size : PSIZE ;
};


// VERTEXSHADERS

vs2ps VS(
    float4 PosO : POSITION ,
    float4 TexCd : TEXCOORD0 ,
    float4 TexCdOnQuad : TEXCOORD1 )

{
    //inititalize all fields of output struct with 0
    
    vs2ps Out = (vs2ps)0;

    //Transform Data from Transform texture
    
    float3 particleTransform = tex2Dlod(SampData, TexCd).xyz;
    
    PosO = mul(PosO, tW);
    PosO.xy += particleTransform.xy;
    Out.Pos = mul(PosO, tVP);
    
    Out.TexCd2 = TexCdOnQuad;
	
    
    if(calcPerspective){
                        Out.Size = particleTransform.z * tP / Out.Pos.w * ViewportHeight/2;
                        }
    else{
         Out.Size = particleTransform.z;
        }
    return Out;
}

// PIXELSHADERS:

float4 PS(vs2ps In): COLOR
{

	float2 textureCoordinate = In.TexCd2;
	float4 col;
	col = tex2D(spriteTextureSamp, textureCoordinate);
    col.xyz *= colorMultiple.xyz;
    col.a *= alfaMultiple;
    return col;
}


// --------------------------------------------------------------------------------------------------
// TECHNIQUES:
// --------------------------------------------------------------------------------------------------

technique ZBufferEnable
{
    pass P0
    {
    	ZEnable = true;
        FillMode = POINT;
        PointScaleEnable = true;
        PointSpriteEnable = true;
    	AlphaRef = 1.000;
    	AlphaFunc = GREATEREQUAL;

        VertexShader = compile vs_3_0 VS();
        PixelShader  = compile ps_3_0 PS();
    }
}
technique ZBufferDisable
{
    pass P0
    {
    	ZEnable = false;
        FillMode = POINT;
        PointScaleEnable = true;
        PointSpriteEnable = true;

        VertexShader = compile vs_3_0 VS();
        PixelShader  = compile ps_3_0 PS();
    }
}